from .grafana_api import GrafanaAPI
from .api import Base


class GrafanaFace:
    def __init__(self, auth, host="localhost", port=None, url_path_prefix="", protocol="http", verify=True):
        self.api = GrafanaAPI(auth, host=host, port=port, url_path_prefix=url_path_prefix, protocol=protocol, verify=verify)

    def __getattr__(self, item):
        for cls in Base.__subclasses__():
            if cls.__name__.lower() == item:
                return cls(api=self.api)
        return



from .base import Base


class Folder(Base):
    def __init__(self, api):
        super().__init__(api)
        self.api = api

    def get_all_folders(self):
        """

        :return:
        """
        path = '/folders'
        r = self.api.GET(path)
        return r

    def get_folder(self, uid):
        """

        :param uid:
        :return:
        """
        path = '/folders/%s' % uid
        r = self.api.GET(path)
        return r

    def create_folder(self, title, uid=None):
        """

        :param title:
        :param uid:
        :return:
        """
        json_data = dict(title=title)
        if uid is not None:
            json_data['uid'] = uid
        return self.api.POST('/folders', json=json_data)

    def update_folder(self, uid, title):
        """

        :param uid:
        :param title:
        :return:
        """
        path = '/folders' % uid
        r = self.api.PUT(path, json={
            "title": title
        })
        return r

    def delete_folder(self, uid):
        """

        :param uid:
        :return:
        """
        path = '/folders/%s' % uid
        r = self.api.DELETE(path)
        return r

    def get_folder_by_id(self, folder_id):
        """

        :param folder_id:
        :return:
        """
        path = '/folders/id/%s' % folder_id
        r = self.api.GET(path)
        return r

    def get_folder_permissions(self):
        """

        :return:
        """
        path = '/folders/%s/permissions'
        r = self.api.GET(path)
        return r

    def update_folder_permissions(self, uid, items):
        """

        :param uid:
        :param items:
        :return:
        """
        update_folder_permissions_path = '/folders/%s/permissions' % uid
        r = self.api.POST(update_folder_permissions_path, json=items)
        return r



from .base import Base


class Search(Base):
    def __init__(self, api):
        super().__init__(api)
        self.api = api

    def search_dashboards(self, query=None, tag=None, type_=None, dashboard_ids=None, folder_ids=None, starred=None,
                          limit=None):
        """

        :param query:
        :param tag:
        :param type_:
        :param dashboard_ids:
        :param folder_ids:
        :param starred:
        :param limit:
        :return:
        """
        list_dashboard_path = '/search'
        params = []

        if query:
            params.append('query=%s' % query)

        if tag:
            params.append('tag=%s' % tag)

        if type_:
            params.append('type=%s' % type_)

        if dashboard_ids:
            params.append('dashboardIds=%s' % dashboard_ids)

        if folder_ids:
            params.append('folderIds=%s' % folder_ids)

        if starred:
            params.append('starred=%s' % starred)

        if limit:
            params.append('limit=%s' % limit)

        list_dashboard_path += '?'
        list_dashboard_path += '&'.join(params)

        r = self.api.GET(list_dashboard_path)
        return r



from .base import Base
from .admin import Admin
from .dashboard import Dashboard
from .datasource import Datasource
from .folder import Folder
from .organisation import Organisation, Organisations
from .search import Search
from .user import User, Users



from .base import Base


class Organisation(Base):
    def __init__(self, api):
        super().__init__(api)
        self.api = api

    def find_organisation(self, org_name):
        """

        :param org_name:
        :return:
        """
        get_org_path = '/orgs/name/%s' % org_name
        r = self.api.GET(get_org_path)
        return r

    def get_current_organisation(self):
        """

        :return:
        """
        get_current_organisation_path = '/org'
        r = self.api.GET(get_current_organisation_path)
        return r

    def create_organisation(self, organisation):
        """

        :param organisation:
        :return:
        """
        create_orgs_path = '/orgs'
        r = self.api.POST(create_orgs_path, json={'name': organisation['name']})
        return r

    def update_current_organisation(self, organisation):
        """

        :param organisation:
        :return:
        """
        update_current_organisation_path = '/org'
        r = self.api.PUT(update_current_organisation_path, json=organisation)
        return r

    def get_current_organisation_users(self):
        """

        :return:
        """
        get_current_organisation_users_path = '/org/users'
        r = self.api.GET(get_current_organisation_users_path)
        return r

    def add_user_current_organisation(self, user):
        """

        :param user:
        :return:
        """
        add_user_current_organisation_path = '/org/users'
        r = self.api.POST(add_user_current_organisation_path, json=user)
        return r

    def update_user_current_organisation(self, user_id, user):
        """

        :param user_id:
        :param user:
        :return:
        """
        update_user_current_organisation_path = '/org/users/%s' % user_id
        r = self.api.PATCH(update_user_current_organisation_path, json=user)
        return r

    def delete_user_current_organisation(self, user_id):
        """

        :param user_id:
        :return:
        """
        delete_user_current_organisation_path = '/org/users/%s' % user_id
        r = self.api.DELETE(delete_user_current_organisation_path)
        return r


class Organisations(Base):
    def __init__(self, api):
        super().__init__(api)
        self.api = api
        self.path = '/users'

    def update_organisation(self, organisation_id, organisation):
        """

        :param organisation_id:
        :param organisation:
        :return:
        """
        update_org_path = '/orgs/%s' % organisation_id
        r = self.api.PUT(update_org_path, json=organisation)
        return r

    def delete_organisation(self, organisation_id):
        """

        :param organisation_id:
        :return:
        """
        delete_org_path = '/orgs/%s' % organisation_id
        r = self.api.DELETE(delete_org_path)
        return r

    def list_organisation(self):
        """

        :return:
        """
        search_org_path = '/orgs'
        r = self.api.GET(search_org_path)
        return r

    def switch_organisation(self, organisation_id):
        """

        :param organisation_id:
        :return:
        """
        switch_user_organisation = '/user/using/%s' % organisation_id
        r = self.api.POST(switch_user_organisation)
        return r

    def organisation_user_list(self, organisation_id):
        """

        :param organisation_id:
        :return:
        """
        users_in_org = '/orgs/%s/users' % organisation_id
        r = self.api.GET(users_in_org)
        return r

    def organisation_user_add(self, organisation_id, user):
        """

        :param organisation_id:
        :param user:
        :return:
        """
        add_user_path = '/orgs/%s/users' % organisation_id
        r = self.api.POST(add_user_path, json=user)
        return r

    def organisation_user_update(self, organisation_id, user_id, user_role):
        """

        :param organisation_id:
        :param user_id:
        :param user_role:
        :return:
        """
        patch_user = '/orgs/%s/users/%s' % (organisation_id, user_id)
        r = self.api.PATCH(patch_user, json={'role': user_role})
        return r

    def organisation_user_delete(self, organisation_id, user_id):
        """

        :param organisation_id:
        :param user_id:
        :return:
        """
        delete_user = '/orgs/%s/users/%s' % (organisation_id, user_id)
        r = self.api.DELETE(delete_user)
        return r

    def organisation_preference_get(self):
        """
        :return:
        """
        update_preference = '/org/preferences'
        r = self.api.GET(update_preference)
        return r

    def organisation_preference_update(self, theme='', home_dashboard_id=0, timezone='utc'):
        """

        :param theme:
        :param home_dashboard_id:
        :param timezone:
        :return:
        """
        update_preference = '/org/preferences'
        r = self.api.PUT(update_preference, json={
            "theme": theme,
            "homeDashboardId": home_dashboard_id,
            "timezone": timezone
        })
        return r



from setuptools import setup, find_packages
from subprocess import check_output


def get_version():
    try:
        tag = check_output(
            ["git", "describe", "--tags", "--abbrev=0", "--match=[0-9]*"]
        )
        return tag.decode('utf-8').strip("\n")
    except Exception:
        raise RuntimeError(
            "The version number cannot be extracted from git tag in this source "
            "distribution; please either download the source from PyPI, or check out "
            "from GitHub and make sure that the git CLI is available."
        )


with open('README.md') as file:
    long_description = file.read()

setup(name='grafana_api',
      version=get_version(),
      description='Yet another Python library for Grafana API',
      long_description=long_description,
      long_description_content_type='text/markdown',
      url='https://github.com/m0nhawk/grafana_api',
      author='Andrew Prokhorenkov',
      author_email='andrew.prokhorenkov@gmail.com',
      license='MIT',
      packages=find_packages(),
      install_requires=[
          'requests',
          'pyyaml',
      ],
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Developers',
          'Topic :: Internet',
          'Topic :: Software Development :: Libraries',
          'Topic :: Software Development :: Libraries :: Python Modules',
          'Operating System :: OS Independent',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
      ],
      zip_safe=False)



import unittest
import sys

if (sys.version_info > (3, 0)):
    from unittest.mock import patch, Mock
else:
    from mock import patch, Mock

import requests

from grafana_api.grafana_face import GrafanaFace
from grafana_api.grafana_api import TokenAuth


class MockResponse:
    def __init__(self, json_data, status_code):
        self.json_data = json_data
        self.status_code = status_code

    def json(self):
        return self.json_data


class TestGrafanaAPI(unittest.TestCase):
    @patch('grafana_api.grafana_api.GrafanaAPI.__getattr__')
    def test_grafana_api(self, mock_get):
        mock_get.return_value = Mock()
        mock_get.return_value.return_value = """{
  "email": "user@mygraf.com",
  "name": "admin",
  "login": "admin",
  "theme": "light",
  "orgId": 1,
  "isGrafanaAdmin": true
}"""
        cli = GrafanaFace(('admin', 'admin'), host='localhost',
                          url_path_prefix='', protocol='https')
        cli.users.find_user('test@test.com')

    def test_grafana_api_no_verify(self):
        cli = GrafanaFace(('admin', 'admin'), host='localhost',
                          url_path_prefix='', protocol='https', verify=False)
        cli.api.s.get = Mock(name='get')
        cli.api.s.get.return_value = MockResponse({
  "email": "user@mygraf.com",
  "name": "admin",
  "login": "admin",
  "theme": "light",
  "orgId": 1,
  "isGrafanaAdmin": True}, 200)

        basicAuth = requests.auth.HTTPBasicAuth('admin', 'admin')
        cli.users.find_user('test@test.com')
        cli.api.s.get.assert_called_once_with('https://localhost/api/users/lookup?loginOrEmail=test@test.com', auth=basicAuth, headers=None, json=None, verify=False)

    def test_grafana_api_basic_auth(self):
        cli = GrafanaFace(('admin', 'admin'), host='localhost',
                          url_path_prefix='', protocol='https')
        self.assertTrue(isinstance(cli.api.auth, requests.auth.HTTPBasicAuth))

    def test_grafana_api_token_auth(self):
        cli = GrafanaFace('alongtoken012345etc', host='localhost',
                          url_path_prefix='', protocol='https')
        self.assertTrue(isinstance(cli.api.auth, TokenAuth))


if __name__ == '__main__':
    import xmlrunner
    unittest.main(testRunner=xmlrunner.XMLTestRunner(output='test-reports'))
